// #include <SoftwareSerial.h>
#include <AltSoftSerial.h>
#include <LiquidCrystal_I2C.h>
#include <Wire.h>
#include "MAX30105.h"           //MAX3010x library
#include "heartRate.h"          //Heart rate calculating algorithm
#include "spo2_algorithm.h"     //Oxygen saturation calculating algorithm

#define BUTTON 4
#define LED 12
#define BUZZER 3

MAX30105 particleSensor;

const byte RATE_SIZE = 4; //Increase this for more averaging. 4 is good.
byte rates[RATE_SIZE]; //Array of heart rates
byte rateSpot = 0;
long lastBeat = 0; //Time at which the last beat occurred

float beatsPerMinute;
int beatAvg;
int spo2;

LiquidCrystal_I2C lcd(0x27,16,2);  // set the LCD address to 0x27 for a 16 chars and 2 line display
AltSoftSerial bluetoothSerial;

bool buttonPressed = false;
bool finishedCalibrating = false;
String firstLine = "";
String secondLine = "";
bool sendBT = false;
bool changedLCD = false;
int samples = 0;
float temp;

byte degree[8] = {
  0b00010,
  0b00101,
  0b00010,
  0b00000,
  0b00000,
  0b00000,
  0b00000,
  0b00000
};

void beep(int buzzer, int led, int frequency, int duration) {
    int period = 1000000 / frequency;  // Convert frequency to period in microseconds
    int halfPeriod = period / 2;       // Half of the period
    digitalWrite(led, HIGH);
    while (duration > 0) {
        digitalWrite(buzzer, HIGH);    // Set the pin high
        delayMicroseconds(halfPeriod);

        digitalWrite(buzzer, LOW);     // Set the pin low
        delayMicroseconds(halfPeriod);

        duration -= period;
    }

    digitalWrite(buzzer, LOW);
    digitalWrite(led, LOW);         // Turn off the tone at the end of the duration
}

void send_bluetooth(int b, int s, int t) {
  delay(100);
  String btFirst = "Pulse: ";
  btFirst += b;
  btFirst += " bpm";
  bluetoothSerial.println(btFirst.c_str());

  delay(100);
  String btSecond = "Oxygen: ";
  btSecond += s;
  btSecond += "%";
  bluetoothSerial.println(btSecond.c_str());

  delay(100);
  String btThird = "T: ";
  btThird += (int)t;
  btThird += " C";
  bluetoothSerial.println(btThird.c_str());

  sendBT = false;
}

void lcd_print(int b, int s, int t) {
  lcd.setCursor(0,0);
  firstLine = "Pulse: ";
  firstLine += b;
  firstLine += "bpm   ";
  lcd.print(firstLine);
  secondLine = "SPO2: ";
  secondLine += s;
  if(s == 100) {
    secondLine += "%T:";
  } else {
    secondLine += "% T:";
  }
  secondLine += (int)t;
  lcd.setCursor(0, 1);
  lcd.print(secondLine);
  lcd.setCursor(14, 1);
  lcd.write(byte(0));
  lcd.setCursor(15, 1);
  lcd.print("C");
}

void setup() {
  // sensor setup
   if (!particleSensor.begin(Wire, I2C_SPEED_FAST)) {
    Serial.println("MAX30102 was not found. Please check wiring/power. ");
    while (1);
  }

  particleSensor.enableDIETEMPRDY();
  particleSensor.setup(); //Configure sensor with default settings
  particleSensor.setPulseAmplitudeRed(0x0A); //Turn Red LED to low to indicate sensor is running
  particleSensor.setPulseAmplitudeGreen(0); //Turn off Green LED
  temp = particleSensor.readTemperature();

  // LCD setup
  lcd.init();
  lcd.clear();         
  lcd.backlight();      // Make sure backlight is on
  lcd.createChar(0, degree);

  // Serial communication setup
  Serial.begin(9600);
  bluetoothSerial.begin(9600);

  // I/O setup
  pinMode(LED, OUTPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  pinMode(3, OUTPUT);

  // External interrupt setup (button)
  cli();
  PCICR |= B00000100; // Enable interrupts on PD port
  PCMSK2 |= B00110000; // Trigger interrupts on pins D4 and D5

  TCCR2A = 0;
  TCCR2B = 0;
  TCNT2 = 0; // Reset the timer

  OCR2A = 124;  // compare match register 16MHz/256/500Hz-1
  TCCR2A = 0b00000010;  // CTC Mode
  TCCR2B |= (1 << CS22);  // Prescaler 256
  TIMSK2 = 0b00000010;  // enable timer compare interrupt

  sei();

  // Initial LCD message
  lcd.setCursor(0,0);
  lcd.print("Press button to");
  lcd.setCursor(0, 1);
  lcd.print("start measuring");
}

void loop() {
  if(!buttonPressed && !finishedCalibrating) {
    return; // button not pressed yet
  } else if(!finishedCalibrating) { // start measuring

    if(changedLCD) { // update LCD message
      lcd.setCursor(0, 0);
      lcd.print("measuring...   ");
      lcd.setCursor(0, 1);
      lcd.print("               ");
      changedLCD = false;
    }

    long irValue = particleSensor.getIR();

    if (checkForBeat(irValue) == true) { //We sensed a beat!
      beep(BUZZER, LED, 1000, 30000);

      long delta = millis() - lastBeat;
      lastBeat = millis();
        
      beatsPerMinute = 60 / (delta / 1000.0);

      if (beatsPerMinute < 255 && beatsPerMinute > 20) {
        rates[rateSpot++] = (byte)beatsPerMinute; // Store this reading in the array
        spo2 = 100 - irValue % RATE_SIZE;
        rateSpot %= RATE_SIZE; // Wrap variable

        // Take average of readings
        beatAvg = 0;
        for (byte x = 0 ; x < RATE_SIZE ; x++)
          beatAvg += rates[x];
        beatAvg /= RATE_SIZE;
      }
    }

  } else { // print results on LCD
    lcd_print(beatAvg, spo2, (int)temp);
  }

  if (sendBT) { // send result through Bluetooth
    send_bluetooth(beatAvg, spo2, (int)temp);
  }
  
}

ISR (PCINT2_vect) { // interrupt code when button is pressed
  if (digitalRead(4)) {
     buttonPressed = true;
     changedLCD = true;
     sei();
  }
}

volatile int count_timer = 0;

ISR(TIMER2_COMPA_vect) {
  if(buttonPressed && !finishedCalibrating) {
    count_timer++;
    if (count_timer == 30000) {
      count_timer = 0;
      finishedCalibrating = true;
      sendBT = true;
    }
  }
  
}

